-- Theme definition of Theseit Technology Preview release 1
-- Author: Park "segfault" J. K. <mastermind!planetmono*org>
--
-- The main purpose of this release is to introduce the fundamental concept of
-- Theseit scripting. Almost everything in Theseit is customizable via the Lua
-- scripting language. Not only in-game, but every graphical elements what you
-- see on screen will take advantage of scripting language.
-- By this, it's possible to make completely different look&feel by modifying
-- script without changing any source codes and compiling them.
-- Please note that this is not the final product and further implementations
-- may vary.
-- For more information about Lua scripting language, visit http://www.lua.org.

------------------------------------------------------------------------------
-- Initialization
-------------------------------------------------------------------------------

skinimage = Image("skin/panel.png") -- Panel image
font = FontFace("skin/custom.ttf")  -- Font face

-- Globals
scrwidth = video.width()
scrheight = video.height()

if game.nchannels() >= 7 then
	channels = 8
else
	channels = 6
end

-- Prepare for textures
function init_texture()
	paneltex = {}
	paneltex.left = Texture(skinimage:copy(0, 0, 8, 600))
	paneltex.right = Texture(skinimage:copy(83, 0, 8, 600))
	paneltex.bottom = Texture(skinimage:copy(8, 550, 75, 50))
	paneltex.chan = {}
	paneltex.note = {}
	paneltex.glow = {}
	paneltex.glow[1] = Texture(skinimage:copy(121, 0, 45, 14))
	paneltex.glow[2] = Texture(skinimage:copy(91, 0, 30, 14))
	paneltex.glow[3] = paneltex.glow[2]
	
	local width = {35, 20, 20}
	local nc = 8
	local nn = 91
	for i = 1, 3 do
		paneltex.chan[i] = Texture(skinimage:copy(nc, 0, width[i], 550))
		paneltex.note[i] = Texture(skinimage:copy(nn, 14, width[i], 4))
		nc = nc + width[i]
		nn = nn + width[i]
	end
end

init_texture()

glowradius = (paneltex.glow[2]:size(0) - paneltex.note[2]:size(0)) / 2

-- Initialize texture objects

function init_objects()
	local buttonmap = {1, 2, 3, 2, 3, 2, 3, 2}
	local sizemap = {}
	local left, width, chanwidth
	
	chanwidth = 0
	for i = 1, channels do
		sizemap[i] = paneltex.chan[buttonmap[i]]:size(0)
		chanwidth = chanwidth + sizemap[i]
	end
	width = chanwidth + paneltex.left:size(0) + paneltex.right:size(0)
	
	objects = {}
	objects.left = TextureObject(paneltex.left)
	objects.right = TextureObject(paneltex.right)
	objects.bottom = TextureObject(paneltex.bottom)
	objects.chan = {}
	objects.note = {}
	objects.glow = {}
	
	left = math.floor((scrwidth / 2) - (width / 2))
	objects.left:setPosition(left, 0)
	left = left + paneltex.left:size(0)
	
	objects.bottom:setSize(chanwidth, 50)
	objects.bottom:setPosition(left, 550)
	
	for i = 1, channels do
		objects.chan[i] = TextureObject(paneltex.chan[buttonmap[i]])
		objects.note[i] = TextureObject(paneltex.note[buttonmap[i]])
		objects.glow[i] = TextureObject(paneltex.glow[buttonmap[i]])
		
		objects.chan[i]:setPosition(left, 0)
		objects.note[i]:setPosition(left, 0)
		objects.glow[i]:setPosition(left - glowradius, -glowradius)
		
		left = left + sizemap[i]
	end
	
	objects.right:setPosition(left, 0)
end

init_objects()

-- BGA
bga = { TextureObject(game.bga(0)), TextureObject(game.bga(1)) }
for i = 1, 2 do
	bga[i]:setSize(800, 600)
end

-- Text
textobj = TextObject()

-- Title text
metadata = {TextObject(), TextObject(), TextObject()}
bars = {Rectangle(scrwidth, 51, 0, 0, 0), Rectangle(scrwidth, 80, 255, 255, 255), Rectangle(scrwidth, 57, 0, 0, 0)}

function generate_title_texture()
	local r = TextRenderer(font, 25, 255, 255, 255)
	r:insertString(game.genre())
	r:render(metadata[1])
	
	r = TextRenderer(font, 48, 0, 0, 0)
	r:insertString(game.title())
	r:render(metadata[2])
	
	r = TextRenderer(font, 30, 255, 255, 255)
	r:insertString(game.artist())
	r:render(metadata[3])
end

generate_title_texture()

if metadata[2]:size(0) > scrwidth-50 then
	metadata[2]:setSizeByAxis(0, scrwidth-50)
end

bars[1]:setPosition(0.0, (scrheight-188) * 0.5)
bars[2]:setPosition(0.0, (scrheight-188) * 0.5 + 51)
bars[3]:setPosition(0.0, (scrheight-188) * 0.5 + 131)
metadata[1]:setPosition(scrwidth*0.5 - metadata[1]:size(0)*0.5, (scrheight-188) * 0.5 + 10)
metadata[2]:setPosition(scrwidth*0.5 - metadata[2]:size(0)*0.5, (scrheight-188) * 0.5 + 61)
metadata[3]:setPosition(scrwidth*0.5 - metadata[3]:size(0)*0.5, (scrheight-188) * 0.5 + 141)

-------------------------------------------------------------------------------
-- Main loop
-------------------------------------------------------------------------------

-- Following functions are essential; they are called automatically by lunaseit.

-- Before starting
triggers = {{0, 1000}, {3000, 1000}, {3500, 750}}
flags = {false, false, false}
blackrect = Rectangle(800, 600, 0, 0, 0)
function pre_game()
	local t = game.timeelapsed()
	
	-- Phase 1
	if t >= triggers[1][1] and flags[1] == false then
		local q = math.sin((t - triggers[1][1]) / triggers[1][2] / 2 * math.pi)*255
		metadata[1]:setAlpha(q)
		bars[2]:setAlpha(q)
		metadata[3]:setAlpha(q)
		
		if t > triggers[1][1] + triggers[1][2] then
			flags[1] = true
			metadata[1]:setAlpha(255)
			bars[2]:setAlpha(255)
			metadata[3]:setAlpha(255)
		end
	end
	
	-- Phase 3; must be rendered prior to title texts
	if t >= triggers[3][1] and flags[3] == false then
		video.orthographicMode()
		
		objects.left:render()
		objects.right:render()
		for i = 1, channels do
			objects.chan[i]:render()
		end
		objects.bottom:render()
		
		blackrect:setAlpha(255 - (math.sin((t - triggers[3][1]) / triggers[3][2] / 2 * math.pi)*255))
		blackrect:render()
		
		if t > triggers[3][1] + triggers[3][2] then
			flags[3] = true
			blackrect:setAlpha(0)
		end
	end
	
	-- Phase 2
	if t >= triggers[2][1] and flags[2] == false then
		local dt = t - triggers[2][1]
		local clamp = math.cos(dt / triggers[2][2] / 2 * math.pi + math.pi) + 1
		local rot = clamp * 90
		local pos = clamp * (scrwidth + 30)
		
		video.perspectiveMode(scrwidth*0.5, scrheight*0.5, scrwidth*0.5, scrwidth*0.5, scrheight*0.5, 0.0, 0.0, 73.75 - dt/500)
		
		metadata[1]:setPositionByAxis(0, (scrwidth*0.5 - metadata[1]:size(0)*0.5) - pos)
		metadata[2]:setPositionByAxis(0, (scrwidth*0.5 - metadata[2]:size(0)*0.5) + pos)
		metadata[3]:setPositionByAxis(0, (scrwidth*0.5 - metadata[3]:size(0)*0.5) - pos)
		bars[1]:setPositionByAxis(0, -pos)
		bars[2]:setPositionByAxis(0, pos)
		bars[3]:setPositionByAxis(0, -pos)
		
		for i = 1, 3 do
			metadata[i]:resetRotation()
			bars[i]:resetRotation()
		end
		metadata[1]:rotate(1, rot)
		metadata[2]:rotate(1, -rot)
		metadata[3]:rotate(1, rot)
		bars[1]:rotate(1, rot)
		bars[2]:rotate(1, -rot)
		bars[3]:rotate(1, rot)
		
		if t > triggers[2][1] + triggers[2][2] then
			flags[2] = true
			video.orthographicMode()
		end
	end
	
	-- Passing condition
	if flags[3] == true then
		game.pass()
	end
	
	-- Render
	for i = 1, 3 do
		bars[i]:render()
		metadata[i]:render()
	end
end

-- Main game loop
lastbpm = -1.0
lastspeed = -1.0
function main_game()
	video.orthographicMode()
	-- Render BGA
	bga[1]:render() -- Lower layer
	bga[2]:render() -- Upper layer
	
	-- Render panel
	objects.left:render()
	objects.right:render()
	for i = 1, channels do
		objects.chan[i]:render()
	end
	
	-- Render notes
	local i = 0
	while true do
		local channel, frontpos, rearpos = game.getNote(i)
		local speed = game.speed()
		local miny = 550.0 - rearpos * speed * 550.0
		local maxy = 550.0 - frontpos * speed * 550.0
		
		local clamp = 1 - (game.measure() * 4 - math.floor(game.measure() * 4))
		
		if maxy < 0 or channel == -1 then break end
		
		if maxy - miny > 0 then
			objects.note[channel]:setSizeByAxis(1, maxy - miny + paneltex.note[1]:size(1))
		else
			objects.note[channel]:setSizeByAxis(1, paneltex.note[1]:size(1))
			objects.glow[channel]:setPositionByAxis(1, miny - paneltex.note[1]:size(1) - glowradius)
			objects.glow[channel]:setAlpha(clamp * clamp * 255)
			objects.glow[channel]:render()
		end
		
		objects.note[channel]:setPositionByAxis(1, miny - paneltex.note[1]:size(1))
		objects.note[channel]:render()
		
		i = i + 1
	end
	
	-- Bottom panel
	objects.bottom:render()
	
	-- Text
	if game.speed() ~= lastspeed or game.bpm() ~= lastbpm then
		lastspeed = game.speed()
		lastbpm = game.bpm()
		local text = TextRenderer(font, 13, 10, 10, 10)
		text:insertString(string.format("%.2fx / %.2f bpm", game.speed(), game.bpm()))
		text:render(textobj)
		textobj:setPosition(math.floor((scrwidth / 2) - (textobj:size(0) / 2)), math.floor(575 - (textobj:size(1) / 2)))
	end
	
	local time = game.timeelapsed()
	if time <= 500 then
		textobj:setAlpha((math.sin((time / 500) * (math.pi * 0.5)) * 127) + 128)
	else
		textobj:setAlpha(255)
	end
	textobj:render()
end

-- Not implemented
function post_game()
	game.pass()
end
